# -*- coding: utf-8 -*-

# Form implementation generated from reading ui file 'image_window_ui.ui'
#
# Created by: PyQt5 UI code generator 5.10.1
#
# WARNING! All changes made in this file will be lost!

from PyQt5 import QtCore, QtGui, QtWidgets

class Ui_ImageDialog(object):
    def setupUi(self, ImageDialog):
        ImageDialog.setObjectName("ImageDialog")
        ImageDialog.resize(791, 641)
        font = QtGui.QFont()
        font.setFamily("Arial")
        ImageDialog.setFont(font)
        self.image = QtWidgets.QWidget(ImageDialog)
        self.image.setGeometry(QtCore.QRect(5, 130, 781, 506))
        self.image.setAutoFillBackground(True)
        self.image.setObjectName("image")
        self.btn_expose = QtWidgets.QPushButton(ImageDialog)
        self.btn_expose.setGeometry(QtCore.QRect(15, 15, 76, 26))
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Ignored, QtWidgets.QSizePolicy.Ignored)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.btn_expose.sizePolicy().hasHeightForWidth())
        self.btn_expose.setSizePolicy(sizePolicy)
        self.btn_expose.setMinimumSize(QtCore.QSize(76, 0))
        font = QtGui.QFont()
        font.setPointSize(10)
        self.btn_expose.setFont(font)
        self.btn_expose.setObjectName("btn_expose")
        self.groupBox_2 = QtWidgets.QGroupBox(ImageDialog)
        self.groupBox_2.setGeometry(QtCore.QRect(710, 10, 71, 106))
        self.groupBox_2.setObjectName("groupBox_2")
        self.btn_strechLow = QtWidgets.QRadioButton(self.groupBox_2)
        self.btn_strechLow.setGeometry(QtCore.QRect(10, 20, 61, 21))
        self.btn_strechLow.setObjectName("btn_strechLow")
        self.btn_strechMid = QtWidgets.QRadioButton(self.groupBox_2)
        self.btn_strechMid.setGeometry(QtCore.QRect(10, 40, 61, 21))
        self.btn_strechMid.setObjectName("btn_strechMid")
        self.btn_strechHigh = QtWidgets.QRadioButton(self.groupBox_2)
        self.btn_strechHigh.setGeometry(QtCore.QRect(10, 60, 61, 21))
        self.btn_strechHigh.setObjectName("btn_strechHigh")
        self.btn_strechSuper = QtWidgets.QRadioButton(self.groupBox_2)
        self.btn_strechSuper.setGeometry(QtCore.QRect(10, 80, 61, 21))
        self.btn_strechSuper.setObjectName("btn_strechSuper")
        self.groupBox_3 = QtWidgets.QGroupBox(ImageDialog)
        self.groupBox_3.setGeometry(QtCore.QRect(630, 10, 76, 106))
        self.groupBox_3.setObjectName("groupBox_3")
        self.btn_size25 = QtWidgets.QRadioButton(self.groupBox_3)
        self.btn_size25.setGeometry(QtCore.QRect(10, 40, 61, 21))
        self.btn_size25.setObjectName("btn_size25")
        self.btn_size50 = QtWidgets.QRadioButton(self.groupBox_3)
        self.btn_size50.setGeometry(QtCore.QRect(10, 60, 61, 21))
        self.btn_size50.setObjectName("btn_size50")
        self.btn_size100 = QtWidgets.QRadioButton(self.groupBox_3)
        self.btn_size100.setGeometry(QtCore.QRect(10, 80, 61, 21))
        self.btn_size100.setObjectName("btn_size100")
        self.btn_size12 = QtWidgets.QRadioButton(self.groupBox_3)
        self.btn_size12.setGeometry(QtCore.QRect(10, 20, 61, 21))
        self.btn_size12.setObjectName("btn_size12")
        self.btn_solve = QtWidgets.QPushButton(ImageDialog)
        self.btn_solve.setGeometry(QtCore.QRect(175, 15, 76, 26))
        font = QtGui.QFont()
        font.setPointSize(10)
        self.btn_solve.setFont(font)
        self.btn_solve.setObjectName("btn_solve")
        self.groupBox_4 = QtWidgets.QGroupBox(ImageDialog)
        self.groupBox_4.setGeometry(QtCore.QRect(530, 10, 96, 106))
        self.groupBox_4.setObjectName("groupBox_4")
        self.btn_colorGrey = QtWidgets.QRadioButton(self.groupBox_4)
        self.btn_colorGrey.setGeometry(QtCore.QRect(10, 20, 71, 21))
        self.btn_colorGrey.setObjectName("btn_colorGrey")
        self.btn_colorCool = QtWidgets.QRadioButton(self.groupBox_4)
        self.btn_colorCool.setGeometry(QtCore.QRect(10, 40, 71, 21))
        self.btn_colorCool.setObjectName("btn_colorCool")
        self.btn_colorRainbow = QtWidgets.QRadioButton(self.groupBox_4)
        self.btn_colorRainbow.setGeometry(QtCore.QRect(10, 60, 81, 21))
        self.btn_colorRainbow.setObjectName("btn_colorRainbow")
        self.btn_colorSpectral = QtWidgets.QRadioButton(self.groupBox_4)
        self.btn_colorSpectral.setGeometry(QtCore.QRect(10, 80, 81, 21))
        self.btn_colorSpectral.setObjectName("btn_colorSpectral")
        self.imageBackground = QtWidgets.QLabel(ImageDialog)
        self.imageBackground.setGeometry(QtCore.QRect(0, 0, 790, 126))
        self.imageBackground.setText("")
        self.imageBackground.setObjectName("imageBackground")
        self.btn_loadFits = QtWidgets.QPushButton(ImageDialog)
        self.btn_loadFits.setEnabled(True)
        self.btn_loadFits.setGeometry(QtCore.QRect(95, 15, 76, 26))
        font = QtGui.QFont()
        font.setPointSize(10)
        self.btn_loadFits.setFont(font)
        self.btn_loadFits.setObjectName("btn_loadFits")
        self.btn_cancel = QtWidgets.QPushButton(ImageDialog)
        self.btn_cancel.setGeometry(QtCore.QRect(15, 85, 76, 26))
        font = QtGui.QFont()
        font.setPointSize(10)
        self.btn_cancel.setFont(font)
        self.btn_cancel.setObjectName("btn_cancel")
        self.le_imageFile = QtWidgets.QLineEdit(ImageDialog)
        self.le_imageFile.setEnabled(False)
        self.le_imageFile.setGeometry(QtCore.QRect(95, 50, 156, 26))
        font = QtGui.QFont()
        font.setFamily("Arial")
        font.setPointSize(10)
        font.setBold(False)
        font.setWeight(50)
        self.le_imageFile.setFont(font)
        self.le_imageFile.setMouseTracking(False)
        self.le_imageFile.setFocusPolicy(QtCore.Qt.NoFocus)
        self.le_imageFile.setAcceptDrops(False)
        self.le_imageFile.setText("")
        self.le_imageFile.setAlignment(QtCore.Qt.AlignLeading|QtCore.Qt.AlignLeft|QtCore.Qt.AlignVCenter)
        self.le_imageFile.setReadOnly(True)
        self.le_imageFile.setObjectName("le_imageFile")
        self.btn_exposeCont = QtWidgets.QPushButton(ImageDialog)
        self.btn_exposeCont.setGeometry(QtCore.QRect(15, 50, 76, 26))
        font = QtGui.QFont()
        font.setPointSize(10)
        self.btn_exposeCont.setFont(font)
        self.btn_exposeCont.setObjectName("btn_exposeCont")
        self.checkShowCrosshairs = QtWidgets.QCheckBox(ImageDialog)
        self.checkShowCrosshairs.setGeometry(QtCore.QRect(105, 90, 131, 16))
        font = QtGui.QFont()
        font.setPointSize(10)
        self.checkShowCrosshairs.setFont(font)
        self.checkShowCrosshairs.setChecked(False)
        self.checkShowCrosshairs.setObjectName("checkShowCrosshairs")
        self.imageMarker = QtWidgets.QWidget(ImageDialog)
        self.imageMarker.setGeometry(QtCore.QRect(5, 130, 781, 506))
        self.imageMarker.setAutoFillBackground(True)
        self.imageMarker.setObjectName("imageMarker")
        self.groupBox = QtWidgets.QGroupBox(ImageDialog)
        self.groupBox.setGeometry(QtCore.QRect(260, 10, 141, 106))
        self.groupBox.setObjectName("groupBox")
        self.le_astrometrySolvingTime = QtWidgets.QLineEdit(self.groupBox)
        self.le_astrometrySolvingTime.setEnabled(False)
        self.le_astrometrySolvingTime.setGeometry(QtCore.QRect(95, 70, 36, 21))
        font = QtGui.QFont()
        font.setFamily("Arial")
        font.setPointSize(10)
        font.setBold(False)
        font.setWeight(50)
        self.le_astrometrySolvingTime.setFont(font)
        self.le_astrometrySolvingTime.setMouseTracking(False)
        self.le_astrometrySolvingTime.setAcceptDrops(False)
        self.le_astrometrySolvingTime.setLayoutDirection(QtCore.Qt.RightToLeft)
        self.le_astrometrySolvingTime.setText("")
        self.le_astrometrySolvingTime.setMaxLength(15)
        self.le_astrometrySolvingTime.setAlignment(QtCore.Qt.AlignRight|QtCore.Qt.AlignTrailing|QtCore.Qt.AlignVCenter)
        self.le_astrometrySolvingTime.setReadOnly(False)
        self.le_astrometrySolvingTime.setObjectName("le_astrometrySolvingTime")
        self.le_astrometryStatusText = QtWidgets.QLineEdit(self.groupBox)
        self.le_astrometryStatusText.setEnabled(False)
        self.le_astrometryStatusText.setGeometry(QtCore.QRect(10, 70, 81, 21))
        font = QtGui.QFont()
        font.setFamily("Arial")
        font.setPointSize(8)
        font.setBold(False)
        font.setWeight(50)
        self.le_astrometryStatusText.setFont(font)
        self.le_astrometryStatusText.setMouseTracking(False)
        self.le_astrometryStatusText.setFocusPolicy(QtCore.Qt.NoFocus)
        self.le_astrometryStatusText.setAcceptDrops(False)
        self.le_astrometryStatusText.setText("")
        self.le_astrometryStatusText.setAlignment(QtCore.Qt.AlignLeading|QtCore.Qt.AlignLeft|QtCore.Qt.AlignVCenter)
        self.le_astrometryStatusText.setReadOnly(True)
        self.le_astrometryStatusText.setObjectName("le_astrometryStatusText")
        self.le_cameraExposureTime = QtWidgets.QLineEdit(self.groupBox)
        self.le_cameraExposureTime.setEnabled(False)
        self.le_cameraExposureTime.setGeometry(QtCore.QRect(95, 30, 36, 21))
        font = QtGui.QFont()
        font.setFamily("Arial")
        font.setPointSize(10)
        font.setBold(False)
        font.setWeight(50)
        self.le_cameraExposureTime.setFont(font)
        self.le_cameraExposureTime.setMouseTracking(False)
        self.le_cameraExposureTime.setAcceptDrops(False)
        self.le_cameraExposureTime.setLayoutDirection(QtCore.Qt.RightToLeft)
        self.le_cameraExposureTime.setText("")
        self.le_cameraExposureTime.setMaxLength(15)
        self.le_cameraExposureTime.setAlignment(QtCore.Qt.AlignRight|QtCore.Qt.AlignTrailing|QtCore.Qt.AlignVCenter)
        self.le_cameraExposureTime.setReadOnly(False)
        self.le_cameraExposureTime.setObjectName("le_cameraExposureTime")
        self.label_84 = QtWidgets.QLabel(self.groupBox)
        self.label_84.setGeometry(QtCore.QRect(10, 15, 81, 16))
        font = QtGui.QFont()
        font.setPointSize(10)
        self.label_84.setFont(font)
        self.label_84.setObjectName("label_84")
        self.label_83 = QtWidgets.QLabel(self.groupBox)
        self.label_83.setGeometry(QtCore.QRect(10, 55, 81, 16))
        font = QtGui.QFont()
        font.setPointSize(10)
        self.label_83.setFont(font)
        self.label_83.setObjectName("label_83")
        self.le_cameraStatusText = QtWidgets.QLineEdit(self.groupBox)
        self.le_cameraStatusText.setEnabled(False)
        self.le_cameraStatusText.setGeometry(QtCore.QRect(10, 30, 81, 21))
        font = QtGui.QFont()
        font.setFamily("Arial")
        font.setPointSize(8)
        font.setBold(False)
        font.setWeight(50)
        self.le_cameraStatusText.setFont(font)
        self.le_cameraStatusText.setMouseTracking(False)
        self.le_cameraStatusText.setFocusPolicy(QtCore.Qt.NoFocus)
        self.le_cameraStatusText.setAcceptDrops(False)
        self.le_cameraStatusText.setText("")
        self.le_cameraStatusText.setAlignment(QtCore.Qt.AlignLeading|QtCore.Qt.AlignLeft|QtCore.Qt.AlignVCenter)
        self.le_cameraStatusText.setReadOnly(True)
        self.le_cameraStatusText.setObjectName("le_cameraStatusText")
        self.groupBox_5 = QtWidgets.QGroupBox(ImageDialog)
        self.groupBox_5.setGeometry(QtCore.QRect(405, 10, 120, 106))
        self.groupBox_5.setObjectName("groupBox_5")
        self.label_86 = QtWidgets.QLabel(self.groupBox_5)
        self.label_86.setGeometry(QtCore.QRect(10, 45, 26, 21))
        font = QtGui.QFont()
        font.setPointSize(10)
        self.label_86.setFont(font)
        self.label_86.setObjectName("label_86")
        self.le_AngleJ2000 = QtWidgets.QLineEdit(self.groupBox_5)
        self.le_AngleJ2000.setEnabled(False)
        self.le_AngleJ2000.setGeometry(QtCore.QRect(55, 70, 51, 21))
        font = QtGui.QFont()
        font.setPointSize(10)
        font.setBold(False)
        font.setWeight(50)
        self.le_AngleJ2000.setFont(font)
        self.le_AngleJ2000.setMouseTracking(False)
        self.le_AngleJ2000.setFocusPolicy(QtCore.Qt.NoFocus)
        self.le_AngleJ2000.setAcceptDrops(False)
        self.le_AngleJ2000.setText("")
        self.le_AngleJ2000.setAlignment(QtCore.Qt.AlignRight|QtCore.Qt.AlignTrailing|QtCore.Qt.AlignVCenter)
        self.le_AngleJ2000.setReadOnly(True)
        self.le_AngleJ2000.setObjectName("le_AngleJ2000")
        self.label_87 = QtWidgets.QLabel(self.groupBox_5)
        self.label_87.setGeometry(QtCore.QRect(10, 70, 36, 21))
        font = QtGui.QFont()
        font.setPointSize(10)
        self.label_87.setFont(font)
        self.label_87.setObjectName("label_87")
        self.le_RaJ2000 = QtWidgets.QLineEdit(self.groupBox_5)
        self.le_RaJ2000.setEnabled(False)
        self.le_RaJ2000.setGeometry(QtCore.QRect(40, 20, 66, 21))
        font = QtGui.QFont()
        font.setPointSize(10)
        font.setBold(False)
        font.setWeight(50)
        self.le_RaJ2000.setFont(font)
        self.le_RaJ2000.setMouseTracking(False)
        self.le_RaJ2000.setFocusPolicy(QtCore.Qt.NoFocus)
        self.le_RaJ2000.setAcceptDrops(False)
        self.le_RaJ2000.setText("")
        self.le_RaJ2000.setAlignment(QtCore.Qt.AlignRight|QtCore.Qt.AlignTrailing|QtCore.Qt.AlignVCenter)
        self.le_RaJ2000.setReadOnly(True)
        self.le_RaJ2000.setObjectName("le_RaJ2000")
        self.le_DecJ2000 = QtWidgets.QLineEdit(self.groupBox_5)
        self.le_DecJ2000.setEnabled(False)
        self.le_DecJ2000.setGeometry(QtCore.QRect(40, 45, 66, 21))
        font = QtGui.QFont()
        font.setFamily("Arial")
        font.setPointSize(10)
        font.setBold(False)
        font.setWeight(50)
        self.le_DecJ2000.setFont(font)
        self.le_DecJ2000.setMouseTracking(False)
        self.le_DecJ2000.setFocusPolicy(QtCore.Qt.NoFocus)
        self.le_DecJ2000.setAcceptDrops(False)
        self.le_DecJ2000.setText("")
        self.le_DecJ2000.setAlignment(QtCore.Qt.AlignRight|QtCore.Qt.AlignTrailing|QtCore.Qt.AlignVCenter)
        self.le_DecJ2000.setReadOnly(True)
        self.le_DecJ2000.setObjectName("le_DecJ2000")
        self.label_85 = QtWidgets.QLabel(self.groupBox_5)
        self.label_85.setGeometry(QtCore.QRect(10, 20, 21, 21))
        font = QtGui.QFont()
        font.setPointSize(10)
        self.label_85.setFont(font)
        self.label_85.setObjectName("label_85")
        self.imageBackground.raise_()
        self.image.raise_()
        self.btn_expose.raise_()
        self.groupBox_2.raise_()
        self.groupBox_3.raise_()
        self.btn_solve.raise_()
        self.groupBox_4.raise_()
        self.btn_loadFits.raise_()
        self.btn_cancel.raise_()
        self.le_imageFile.raise_()
        self.btn_exposeCont.raise_()
        self.checkShowCrosshairs.raise_()
        self.imageMarker.raise_()
        self.groupBox.raise_()
        self.groupBox_5.raise_()

        self.retranslateUi(ImageDialog)
        QtCore.QMetaObject.connectSlotsByName(ImageDialog)

    def retranslateUi(self, ImageDialog):
        _translate = QtCore.QCoreApplication.translate
        ImageDialog.setWindowTitle(_translate("ImageDialog", "Imaging"))
        self.btn_expose.setToolTip(_translate("ImageDialog", "<html><head/><body><p>Single exposure</p></body></html>"))
        self.btn_expose.setText(_translate("ImageDialog", "Expose 1"))
        self.groupBox_2.setTitle(_translate("ImageDialog", "Strech"))
        self.btn_strechLow.setText(_translate("ImageDialog", "Low"))
        self.btn_strechMid.setText(_translate("ImageDialog", "Mid"))
        self.btn_strechHigh.setText(_translate("ImageDialog", "High"))
        self.btn_strechSuper.setText(_translate("ImageDialog", "Super"))
        self.groupBox_3.setTitle(_translate("ImageDialog", "Zoom"))
        self.btn_size25.setText(_translate("ImageDialog", "4x"))
        self.btn_size50.setText(_translate("ImageDialog", "2x"))
        self.btn_size100.setText(_translate("ImageDialog", "1x"))
        self.btn_size12.setText(_translate("ImageDialog", "8x"))
        self.btn_solve.setToolTip(_translate("ImageDialog", "<html><head/><body><p>Single plate solve of the actual image</p></body></html>"))
        self.btn_solve.setText(_translate("ImageDialog", "Solve"))
        self.groupBox_4.setTitle(_translate("ImageDialog", "Colors"))
        self.btn_colorGrey.setToolTip(_translate("ImageDialog", "<html><head/><body><p>Color scheme black /white</p></body></html>"))
        self.btn_colorGrey.setText(_translate("ImageDialog", "Grey"))
        self.btn_colorCool.setToolTip(_translate("ImageDialog", "<html><head/><body><p>Color scheme red/blue</p></body></html>"))
        self.btn_colorCool.setText(_translate("ImageDialog", "Cool"))
        self.btn_colorRainbow.setToolTip(_translate("ImageDialog", "<html><head/><body><p>Color scheme rainbow</p></body></html>"))
        self.btn_colorRainbow.setText(_translate("ImageDialog", "Rainbow"))
        self.btn_colorSpectral.setToolTip(_translate("ImageDialog", "<html><head/><body><p>Color scheme rainbow</p></body></html>"))
        self.btn_colorSpectral.setText(_translate("ImageDialog", "Spectral"))
        self.imageBackground.setProperty("color", _translate("ImageDialog", "blue"))
        self.btn_loadFits.setToolTip(_translate("ImageDialog", "<html><head/><body><p><span style=\" font-size:10pt;\">Load a fits file and display is</span></p></body></html>"))
        self.btn_loadFits.setText(_translate("ImageDialog", "Load FITS"))
        self.btn_cancel.setToolTip(_translate("ImageDialog", "<html><head/><body><p>Cancels an imaging or plate solving action or stops continous exposures</p></body></html>"))
        self.btn_cancel.setText(_translate("ImageDialog", "Cancel"))
        self.le_imageFile.setToolTip(_translate("ImageDialog", "<html><head/><body><p>name of image which is shown</p></body></html>"))
        self.btn_exposeCont.setToolTip(_translate("ImageDialog", "<html><head/><body><p><span style=\" font-size:10pt;\">Continous exposures</span></p></body></html>"))
        self.btn_exposeCont.setText(_translate("ImageDialog", "Expose N"))
        self.checkShowCrosshairs.setToolTip(_translate("ImageDialog", "<html><head/><body><p>Show crosshairs on image</p></body></html>"))
        self.checkShowCrosshairs.setText(_translate("ImageDialog", "Show crosshairs"))
        self.groupBox.setTitle(_translate("ImageDialog", "Device Stats"))
        self.le_astrometrySolvingTime.setToolTip(_translate("ImageDialog", "<html><head/><body><p>Time elapsed for plate solving</p></body></html>"))
        self.le_astrometryStatusText.setToolTip(_translate("ImageDialog", "<html><head/><body><p><span style=\" font-size:10pt;\">Status feedback from astrometry</span></p></body></html>"))
        self.le_cameraExposureTime.setToolTip(_translate("ImageDialog", "<html><head/><body><p>Time left for image integration</p></body></html>"))
        self.label_84.setText(_translate("ImageDialog", "Camera"))
        self.label_83.setText(_translate("ImageDialog", "Astrometry"))
        self.le_cameraStatusText.setToolTip(_translate("ImageDialog", "<html><head/><body><p><span style=\" font-size:10pt;\">Status feedback from camera</span></p></body></html>"))
        self.groupBox_5.setTitle(_translate("ImageDialog", "Solve Result"))
        self.label_86.setText(_translate("ImageDialog", "DEC"))
        self.le_AngleJ2000.setToolTip(_translate("ImageDialog", "<html><head/><body><p>Shows the solved angle of image in degrees</p></body></html>"))
        self.label_87.setText(_translate("ImageDialog", "Angle"))
        self.le_RaJ2000.setToolTip(_translate("ImageDialog", "<html><head/><body><p><span style=\" font-weight:400;\">Shows the solved RA of image in J2000 coordinates</span></p></body></html>"))
        self.le_DecJ2000.setToolTip(_translate("ImageDialog", "<html><head/><body><p><span style=\" font-weight:400;\">Shows the solved DEC of image in J2000 coordinates</span></p></body></html>"))
        self.label_85.setText(_translate("ImageDialog", "RA"))

